// ESP32 Touch Test
// Just test touch pin - Touch0 is T0 which is on GPIO 4.


//#define BLE_MOUSE 1
#define DEBUG 1

#ifdef BLE_MOUSE
#include <BleMouse.h>

BleMouse bleMouse;
#endif

int touch[8] = {13, 14, 33, 15, 4, 12, 32, 27};
int value[8] = {0, 0, 0, 0, 0, 0, 0, 0};


int buttonGPIO[4] = {36, 39, 35, 34};
int buttonState[4] = {0, 0, 0, 0};

const int ledPin = 26;


void setup() {
  Serial.begin(115200);
  delay(1000); // give me time to bring up serial monitor
  Serial.println("ESP32 Touch BLE Mouse");

  for (int i = 0; i < 4; ++i) {
    pinMode(buttonGPIO[i], INPUT);
  }

  // LED
  pinMode(ledPin, OUTPUT);
  digitalWrite(ledPin, HIGH);

#ifdef BLE_MOUSE
  bleMouse.begin("Finger Pan Mouse");
#endif
}

int lastPoll = 0;
const int touchThreshold = 15;
int ring[2] = {0, 0};
int prevRing[2] = {0, 0};
int mode = 0;
int clickState[2] = {0, 0};
int ringTime[6] = {0, 0, 0, 0, 0, 0};
int speed = 1;

void loop() {
  int move[2] = {0, 0};
  int tms = millis();
  if (tms > (lastPoll + 50)) {
    for (int i = 0; i < 8; ++i) {
      int t = touchRead(touch[i]);
      int d = t - value[i];
      value[i] = t;
      if ((d < -touchThreshold) || (d > touchThreshold)) {
        int j = i >> 2;
        int k = i & 3;
        if (d < 0) {
          // pad touched
          ring[j] |= (1 << k);
        } else {
          // untouched
          ring[j] &= ~(1 << k);
        }
        int r = ring[j] & 7;
        if (r != prevRing[j]) {
          if ((r == 1) || (r == 2) || (r == 4)) {
            int k0 = (r - 1) - ((r == 4) ? 1 : 0);
            ringTime[j * 3 + k0] = tms;
            int k1 = ((k0 + 1) % 3) + 3 * j;
            int k2 = ((k0 + 2) % 3) + 3 * j;
            int dir = 1;
            int rt = ringTime[k2];
            if (ringTime[k1] > ringTime[k2]) {
              dir = -1;
              rt = ringTime[k1];
            }
            if (rt > (tms - 500)) {
              if (rt < tms) {
                move[j] = dir * speed;
                if (speed < (5000 / (tms - rt))) {
                  speed += 4;
                }
              }
            }
          }
          prevRing[j] = r;
        }
      }
    }

    for (int j = 0; j < 2; ++j) {
      clickState[j] = ((clickState[j] << 1) & 3) | ((ring[j] == 8) ? 1 : 0);
    }
  
    for (int i = 0; i < 4; ++i) {
      int button = digitalRead(buttonGPIO[i]);
      buttonState[i] = ((buttonState[i] << 1) & 0xFFF) | ((button == HIGH) ? 1 : 0);
    }

    if ((buttonState[0] & 3) == 1) {
      mode = 0;
    }
    if ((buttonState[1] & 3) == 1) {
      mode = 1;
    }

    if (speed > 1) {
      --speed;
    }
    lastPoll = tms;
#ifdef DEBUG
    Serial.print(clickState[0]);
    Serial.print(" ");
    Serial.print(clickState[1]);
    Serial.print(" ");
    Serial.print(move[0]);
    Serial.print(",");
    Serial.println(move[1]);
#endif
  }

#ifdef BLE_MOUSE
  if (bleMouse.isConnected()) {
    digitalWrite(ledPin, LOW);
    if (lastPoll == tms) {
      if ((clickState[0] & 3) == 1) {
        Serial.println("MOUSE_LEFT");
        bleMouse.click(MOUSE_LEFT);
      }
      if ((clickState[1] & 3) == 1) {
        Serial.println("MOUSE_RIGHT");
        bleMouse.click(MOUSE_RIGHT);
      }
      ble.move(move[0], move[1]);
    }
  }
#endif
}
